import globalUI from '@sinoform/global-ui';
import { PageDirType, PageSizeType } from '@sinoform/types';
import { produce } from 'immer';
import { BorderType } from '../constants';
import getAllSelectedCells from '../utils/getAllSelectedCells';
import { canPastCell, checkCellRange } from './helper/checkCellRange';
import getCellValueInRange from './helper/getCellValueInRange';
import { ActionType } from './types';
export const defaultState = {
    cols: {
        len: 10,
    },
    rows: {
        len: 80,
    },
    config: {
        pageConfig: {
            pageDir: PageDirType.VERTICAL,
            pageSize: PageSizeType.A4,
            pagePaddingTop: '25.5',
            pagePaddingBottom: '25.5',
            pagePaddingLeft: '23.7',
            pagePaddingRight: '23.7',
        },
        contentConfig: {
            isAutoHeight: '0',
            isShowDefaultTitle: '0',
        },
    },
};
/**
 * 合并或拆分单元格
 *
 * @param draft
 * @param action
 */
const mergeCells = (draft, action) => {
    const { fromCell, toCell } = action.payload;
    // 判断起始和结束范围内的单元格有没有被合并的单元格，如果存在执行拆分动作，否则执行合并动作
    const cells = (draft.merges ?? []).filter(([[sri, sci], [eri, eci]]) => sri >= fromCell[0] &&
        sci >= fromCell[1] &&
        eri <= toCell[0] &&
        eci <= toCell[1]);
    if (cells.length > 0) {
        draft.merges = draft.merges?.filter(([[sri, sci], [eri, eci]]) => !(sri >= fromCell[0] &&
            sci >= fromCell[1] &&
            eri <= toCell[0] &&
            eci <= toCell[1]));
        // eslint-disable-next-line no-restricted-syntax
        for (const cell of cells) {
            if (draft.rows[cell[0][0]]?.cells?.[cell[0][1]]) {
                delete draft.rows[cell[0][0]].cells[cell[0][1]].merge;
            }
        }
    }
    else {
        draft.merges = draft.merges ?? [];
        draft.merges.push([fromCell, toCell]);
        const cellValues = getCellValueInRange([fromCell, toCell], draft);
        for (let i = fromCell[0]; i <= toCell[0]; i += 1) {
            for (let j = fromCell[1]; j <= toCell[1]; j += 1) {
                if (i === fromCell[0] && j === fromCell[1]) {
                    draft.rows[i] = draft.rows[i] ?? { cells: {} };
                    draft.rows[i].cells = draft.rows[i].cells ?? {};
                    draft.rows[i].cells[j] = draft.rows[i].cells?.[j] ?? {};
                    draft.rows[i].cells[j].merge = [
                        toCell[0] - fromCell[0],
                        toCell[1] - fromCell[1],
                    ];
                    // 如果选中区域只有一个有效值，直接使用该值，否则使用左上角单元格的值
                    draft.rows[i].cells[j].value =
                        cellValues.length === 1
                            ? cellValues[0]
                            : draft.rows[i].cells[j].value;
                }
                else if (draft.rows?.[i]?.cells?.[j]) {
                    delete draft.rows[i].cells[j];
                }
            }
        }
    }
};
/**
 * 改变行高
 *
 * @param draft
 * @param action
 */
const changeRowHeight = (draft, action) => {
    const { rowIdx, height } = action.payload;
    draft.rows[rowIdx] = draft.rows[rowIdx] ?? {};
    draft.rows[rowIdx].height = height;
};
/*
 * 插入行
 * @param draft
 * @param action
 */
const insertRow = (draft, action) => {
    const { payload } = action;
    draft.rows.len += 1;
    // 处理单元格的合并信息
    (draft.merges ?? []).forEach((merge, idx) => {
        const [[sri, sci], [tri, tci]] = merge;
        // 如果插入行横穿合并的单元格，则合并结束单元格索引增加1
        if (sri < payload && tri >= payload) {
            draft.rows[sri].cells[sci] = {
                ...draft.rows[sri].cells[sci],
                merge: [
                    draft.rows[sri].cells[sci].merge[0] + 1,
                    draft.rows[sri].cells[sci].merge[1],
                ],
            };
            draft.merges[idx] = [merge[0], [tri + 1, tci]];
        }
        // 如果合并单元格在插入位置之后，则合并单元格的起始行和结束行索引均增加1
        if (sri >= payload) {
            draft.merges[idx] = [
                [sri + 1, sci],
                [tri + 1, tci],
            ];
        }
    });
    const newRows = {
        len: draft.rows.len,
    };
    // 处理每行的单元格数据
    Object.entries(draft.rows)
        .filter((item) => item[0] !== 'len')
        .forEach(([rowIdx, row]) => {
        const currentRowIdx = parseInt(rowIdx, 10);
        if (currentRowIdx < payload) {
            // 如果当前行在插入行之前，则不做处理，否则行索引增加1
            newRows[currentRowIdx] = row;
        }
        else {
            newRows[currentRowIdx + 1] = row;
        }
    });
    draft.rows = newRows;
};
/**
 * 从制定位置删除指定行数
 *
 * @param draft 状态数据
 * @param action 动作
 */
const deleteRow = (draft, action) => {
    const { payload: { from, len }, } = action;
    const end = from + len - 1;
    // 处理总行数
    draft.rows.len -= len;
    const formRowCells = {};
    // 处理单元格合并集合数据
    (draft.merges ?? []).forEach((merge, idx) => {
        const [[sri, sci], [tri, tci]] = merge;
        // 合并的单元格整个在删除区域内,删除此单元格及单元格对应的合并数据
        if (sri >= from && tri <= end) {
            delete draft.rows[sri].cells[sci];
            delete draft.merges[idx];
        }
        // 合并的单元格的起始行在删除区域内，但结束行不在
        if (sri >= from && sri <= end && tri > end) {
            formRowCells[sci] = {
                ...draft.rows[sri].cells[sci],
                value: undefined,
                merge: [
                    draft.rows[sri].cells[sci].merge[0] - (end - sri + 1),
                    draft.rows[sri].cells[sci].merge[1],
                ],
            };
            draft.merges[idx] = [
                [from, sci],
                [tri - len, tci],
            ];
        }
        // 合并的单元格的起始行不在删除区域，但结束行在删除区域
        if (sri < from && tri >= from && tri < end) {
            draft.rows[sri].cells[sci] = {
                ...draft.rows[sri].cells[sci],
                merge: [
                    draft.rows[sri].cells[sci].merge[0] - (tri - from + 1),
                    draft.rows[sri].cells[sci].merge[1],
                ],
            };
            draft.merges[idx] = [
                [sri, sci],
                [from - 1, tci],
            ];
        }
        // 合并单元格在删除行之后的行
        if (sri > end) {
            draft.merges[idx] = [
                [sri - len, sci],
                [tri - len, tci],
            ];
        }
        // 如果删除区域横穿合并区域，则只需把合并结束行索引减去删除的长度即可
        if (sri < from && tri >= end) {
            draft.rows[sri].cells[sci] = {
                ...draft.rows[sri].cells[sci],
                merge: [
                    draft.rows[sri].cells[sci].merge[0] - len,
                    draft.rows[sri].cells[sci].merge[1],
                ],
            };
            draft.merges[idx] = [
                [sri, sci],
                [tri - len, tci],
            ];
        }
    });
    draft.merges = draft.merges?.filter(Boolean);
    const newRows = {
        len: draft.rows.len,
    };
    // 处理每行的单元格数据
    Object.entries(draft.rows)
        .filter((item) => item[0] !== 'len')
        .forEach(([rowIdx, row]) => {
        const currentRowIdx = parseInt(rowIdx, 10);
        if (currentRowIdx < from) {
            newRows[currentRowIdx] = row;
        }
        else if (currentRowIdx > end) {
            newRows[currentRowIdx - len] = row;
        }
    });
    if (Object.keys(formRowCells).length > 0) {
        newRows[from] = newRows[from] ?? { cells: {} };
        Object.assign(newRows[from].cells, formRowCells);
    }
    draft.rows = newRows;
};
/**
 * 改变列宽
 *
 * @param draft
 * @param action
 */
const changeColWidth = (draft, action) => {
    const { colIdx, width } = action.payload;
    draft.cols[colIdx] = draft.cols[colIdx] ?? {};
    draft.cols[colIdx].width = width;
};
/**
 * 插入列
 *
 * 选中列的前面插入新的列
 *
 * 几种场景：
 * - 新插入的列穿过了合并的单元格
 * - 合并的单元格在新插入的列后面
 * - 合并的单元格在新插入的列前面
 *
 * 要修改的数据：
 * 1. 修改总列数
 * 2. 对穿透的单元格进行合并列+1
 * 3. 对插入列之后的所有单元格列下标+1
 * 4. 修改全部的合并单元格
 *
 * @param draft
 * @param action
 */
const insertCol = (draft, action) => {
    const insertIdx = action.payload;
    // 修改总列数
    draft.cols.len += 1;
    // 修改合并单元格对象
    draft.merges = draft.merges?.map(([[sri, sci], [eri, eci]]) => {
        if (sci >= insertIdx) {
            // 新插入列之后的合并单元格
            return [
                [sri, sci + 1],
                [eri, eci + 1],
            ];
        }
        if (sci < insertIdx && eci >= insertIdx) {
            // 新插入的列穿过合并的单元格
            return [
                [sri, sci],
                [eri, eci + 1],
            ];
        }
        return [
            [sri, sci],
            [eri, eci],
        ];
    });
    // 处理单元格
    Object.entries(draft.rows)
        .filter((item) => item[0] !== 'len')
        .forEach(([rowIdx, row]) => {
        const { cells } = row;
        const rIdx = parseInt(rowIdx, 10);
        const newCellValue = {};
        Object.entries(cells ?? {})
            .filter((item) => item[0] !== 'height')
            .forEach(([colIdx, cell]) => {
            const { merge = [0, 0] } = cell;
            const cIdx = parseInt(colIdx, 10);
            if (cIdx + merge[1] < insertIdx) {
                // 没有影响
                newCellValue[cIdx] = cell;
            }
            else if (merge[1] + cIdx >= insertIdx && cIdx < insertIdx) {
                // 新增的列穿过了合并的单元格
                newCellValue[cIdx] = { ...cell, merge: [merge[0], merge[1] + 1] };
            }
            else {
                // 否则将影响到的单元格所在列+1
                newCellValue[cIdx + 1] = cell;
            }
        });
        draft.rows[rIdx].cells = newCellValue;
    });
};
/**
 * 删除列，支持选中多列同时删除
 *
 * @param draft
 * @param action
 */
const deleteCol = (draft, action) => {
    const { startCol, endCol } = action.payload;
    const removeCols = endCol - startCol + 1;
    // 修改总列数
    draft.cols.len -= removeCols;
    // 修改总的合并单元格数据
    draft.merges = draft.merges
        ?.map(([[sri, sci], [eri, eci]]) => {
        if (eci < startCol) {
            // 合并的单元格在删除的列之前不受影响
            return [
                [sri, sci],
                [eri, eci],
            ];
        }
        if (eci >= startCol && eci <= endCol && sci < startCol) {
            // 合并的单元格的末尾行在删除列中
            return [
                [sri, sci],
                [eri, startCol - 1],
            ];
        }
        if (sci >= startCol && sci <= endCol && eci > endCol) {
            // 合并的单元格的起始行在删除列中
            return [
                [sri, startCol],
                [eri, eci - (sci - startCol + 1)],
            ];
        }
        if (sci >= startCol && eci <= endCol) {
            //  合并的单元格完整在删除列中
            return undefined;
        }
        if (sci < startCol && eci > endCol) {
            // 删除列穿过合并的单元格
            return [
                [sri, sci],
                [eri, eci - removeCols],
            ];
        }
        // 合并的单元格在删除的列之后，减去删除的列数
        return [
            [sri, sci - removeCols],
            [eri, eci - removeCols],
        ];
    })
        ?.filter(Boolean)
        ?.filter(([[sri, sci], [eri, eci]]) => !(eri - sri === 0 && eci - sci === 0));
    // 处理单元格
    Object.entries(draft.rows)
        .filter((item) => item[0] !== 'len')
        .forEach(([rowIdx, row]) => {
        const { cells } = row;
        const rIdx = parseInt(rowIdx, 10);
        const newCellValue = {};
        Object.entries(cells ?? {})
            .filter((item) => item[0] !== 'height')
            .forEach(([colIdx, cell]) => {
            const { merge = [0, 0] } = cell;
            const cIdx = parseInt(colIdx, 10);
            if (cIdx + merge[1] < startCol) {
                // 单元格在删除列之前
                newCellValue[cIdx] = cell;
            }
            else if (cIdx + merge[1] >= startCol &&
                cIdx + merge[1] <= endCol &&
                cIdx < startCol) {
                // 合并单元格的末尾列在删除列之中
                newCellValue[cIdx] = {
                    ...cell,
                    merge: [merge[0], startCol - cIdx - 1],
                };
            }
            else if (cIdx >= startCol &&
                cIdx <= endCol &&
                cIdx + merge[1] > endCol) {
                // 合并的单元格起始列在删除列之中
                newCellValue[startCol] = {
                    ...cell,
                    merge: [merge[0], cIdx + merge[1] - endCol - 1],
                    value: undefined,
                };
            }
            else if (cIdx >= startCol && cIdx + merge[1] <= endCol) {
                // 单元格在删除列中，直接删除，不做操作
            }
            else if (cIdx < startCol && cIdx + merge[1] > endCol) {
                // 删除列穿过合并的单元格
                newCellValue[cIdx] = {
                    ...cell,
                    merge: [merge[0], merge[1] - removeCols],
                };
            }
            else {
                // 合并的单元格在删除的列之后，减去删除的列数
                newCellValue[cIdx - removeCols] = cell;
            }
        });
        draft.rows[rIdx].cells = newCellValue;
    });
};
/**
 * 单元格拷贝
 *
 * 需要判断复制的单元格是否允许拷贝到目标单元格
 *
 * 简化规则：
 * - 单个单元格的拷贝：
 *   - 如果是普通的单元格直接拷贝内容到目标单元格
 *   - 如果是合并的单元格，目标单元格也是合并的单元格，直接拷贝内容
 *   - 如果是合并的单元格，目标单元格是普通的单元格，需要判定目标单元格旁边的单元格是否满足按照被拷贝的单元格的方式进行单元格合并，不满足不进行拷贝动作
 * - 多个单元格的拷贝：
 *   - 判定源单元格对应的目标区域的单元格是否满足复制条件，及源单元格的覆盖区域移动到对应的目标区域之后，是否有单元格（肯定是合并的单元格）超出目标区域
 *
 * @param draft
 * @param action
 */
const copyCell = (draft, action) => {
    const { source: [[sri, sci], [eri, eci]], target: [tri, tci], } = action.payload;
    const { merges } = draft;
    if (!canPastCell([
        [sri, sci],
        [eri, eci],
    ], [tri, tci], draft)) {
        globalUI.message.error('单元格复制目标区域超出边界');
        return;
    }
    //  单个单元格
    if ((eri === sri && eci === sci) ||
        merges?.some(([[msri, msci], [meri, meci]]) => msri === sri && meri === eri && msci === sci && meci === eci)) {
        const sourceMerge = draft.rows[sri]?.cells?.[sci]?.merge ?? [0, 0];
        const targetMerge = draft.rows[tri]?.cells?.[tci]?.merge ?? [0, 0];
        if (
        // 被拷贝的单元格为普通的单元格
        (sourceMerge[0] === 0 && sourceMerge[1] === 0) ||
            // 被拷贝的单元格与目标单元格均为合并的单元格
            ((sourceMerge[0] !== 0 || sourceMerge[1] !== 0) &&
                (targetMerge[0] !== 0 || targetMerge[1] !== 0))) {
            draft.rows[tri] = draft.rows[tri] ?? { cells: {} };
            draft.rows[tri].cells[tci] = draft.rows[tri].cells[tci] ?? {};
            draft.rows[tri].cells[tci].value = draft.rows[sri]?.cells?.[sci]?.value;
            return;
        }
    }
    if (!checkCellRange([tri, tci], [
        [sri, sci],
        [eri, eci],
    ], merges)) {
        globalUI.message.error('无法对合并的单元格执行此操作');
        return;
    }
    // 处理merges
    const targetMerges = merges
        ?.filter(([[msri, msci], [meri, meci]]) => msri >= sri && meri <= eri && msci >= sci && meci <= eci)
        ?.map(([[msri, msci], [meri, meci]]) => [
        [msri + tri - sri, msci + tci - sci],
        [meri + tri - sri, meci + tci - sci],
    ]) ?? [];
    draft.merges = merges?.filter(([[msri, msci], [meri, meci]]) => !(msri >= tri &&
        meri <= tri + eri - sri &&
        msci >= tci &&
        meci <= tci + eci - sci));
    draft.merges?.push(...targetMerges);
    // 处理单元格
    for (let i = sri; i <= eri; i += 1) {
        for (let j = sci; j <= eci; j += 1) {
            if (draft.rows[i]?.cells?.[j]) {
                // 源单元格存在数据，直接复制到目标单元格
                draft.rows[tri + i - sri] = draft.rows[tri + i - sri] ?? {
                    cells: {},
                };
                draft.rows[tri + i - sri].cells[tci + j - sci] = draft.rows[i].cells[j];
            }
            else if (draft.rows[tri + i - sri]?.cells?.[tci + j - sci]) {
                // 如果源位置不存在单元格直接删除
                delete draft.rows[tri + i - sri].cells[tci + j - sci];
            }
        }
    }
};
/**
 * 编辑单元格
 *
 * 单元格编辑有两个地方触发：
 * - 手动编辑单元格时
 * - 拖拽添加表单项时
 *
 * 这里的处理是按照单元格内容是完整可用的可直接替换的处理
 *
 * @param draft
 * @param action
 */
const editCellValue = (draft, action) => {
    const { cellPos: [rowIdx, colIdx], value, } = action.payload;
    if (rowIdx < 0 || colIdx < 0)
        return;
    draft.rows[rowIdx] = draft.rows[rowIdx] ?? { cells: {} };
    draft.rows[rowIdx].cells = draft.rows[rowIdx].cells ?? {};
    draft.rows[rowIdx].cells[colIdx] = {
        ...draft.rows[rowIdx].cells[colIdx],
        value,
    };
};
/**
 * 设置单元格的样式，包括单个单元格和多个单元格
 *
 * 这里的样式主要包括：
 * - 单元格边框线
 * - 水平布局
 * - 垂直布局
 *
 * @param draft
 * @param action
 */
const setCellsStyle = (draft, action) => {
    const { cells, style } = action.payload;
    cells.forEach(([rowIdx, colIdx]) => {
        draft.rows[rowIdx] = draft.rows[rowIdx] ?? { cells: {} };
        draft.rows[rowIdx].cells[colIdx] = draft.rows[rowIdx].cells[colIdx] ?? {};
        draft.rows[rowIdx].cells[colIdx] = {
            ...draft.rows[rowIdx].cells[colIdx],
            style: { ...draft.rows[rowIdx].cells[colIdx].style, ...style },
        };
    });
};
/**
 * 设置单元格文本样式，主要包括以下样式
 *
 * - 字体
 * - 字号
 * - 颜色
 * - 背景色
 * - 加粗
 * - 倾斜
 * - 下划线
 *
 * @param draft
 * @param action
 */
const setCellsTextStyle = (draft, action) => {
    const { cells, items } = action.payload;
    cells.forEach(([rowIdx, colIdx], idx) => {
        draft.rows[rowIdx] = draft.rows[rowIdx] ?? { cells: {} };
        draft.rows[rowIdx].cells[colIdx] = {
            ...draft.rows[rowIdx].cells[colIdx],
            value: items[idx], // 设置单元格文本样式实际是与slatejs交互变更的是单元格的value
        };
    });
};
/**
 * 单元格剪切
 *
 * @param draft
 * @param action
 * @returns
 */
const cutCell = (draft, action) => {
    const { source: [[sri, sci], [eri, eci]], target: [tri, tci], } = action.payload;
    const { merges } = draft;
    if (!canPastCell([
        [sri, sci],
        [eri, eci],
    ], [tri, tci], draft)) {
        globalUI.message.error('单元格复制目标区域超出边界');
        return;
    }
    if (!checkCellRange([tri, tci], [
        [sri, sci],
        [eri, eci],
    ], merges)) {
        globalUI.message.error('无法对合并的单元格执行此操作');
        return;
    }
    // 处理merges
    const targetMerges = merges
        ?.filter(([[msri, msci], [meri, meci]]) => msri >= sri && meri <= eri && msci >= sci && meci <= eci)
        ?.map(([[msri, msci], [meri, meci]]) => [
        [msri + tri - sri, msci + tci - sci],
        [meri + tri - sri, meci + tci - sci],
    ]) ?? [];
    draft.merges = merges?.filter(([[msri, msci], [meri, meci]]) => 
    // 目标区域的单元格合并信息
    !(msri >= tri &&
        meri <= tri + eri - sri &&
        msci >= tci &&
        meci <= tci + eci - sci) &&
        // 源单元格区域
        !(msri >= sri && meri <= eri && msci >= sci && meci <= eci));
    draft.merges?.push(...targetMerges);
    // 处理单元格
    for (let i = sri; i <= eri; i += 1) {
        for (let j = sci; j <= eci; j += 1) {
            if (draft.rows[i]?.cells?.[j]) {
                // 源单元格存在数据，直接复制到目标单元格
                draft.rows[tri + i - sri] = draft.rows[tri + i - sri] ?? {
                    cells: {},
                };
                draft.rows[tri + i - sri].cells[tci + j - sci] = draft.rows[i].cells[j];
                delete draft.rows[i].cells[j];
            }
            else if (draft.rows[tri + i - sri]?.cells?.[tci + j - sci]) {
                // 如果源位置不存在单元格直接删除
                delete draft.rows[tri + i - sri].cells[tci + j - sci];
            }
        }
    }
};
const ensureCellStyle = (draft, rowIdx, colIdx) => {
    draft.rows[rowIdx] = draft.rows[rowIdx] ?? { cells: {} };
    draft.rows[rowIdx].cells[colIdx] = draft.rows[rowIdx].cells[colIdx] ?? {};
    draft.rows[rowIdx].cells[colIdx].style =
        draft.rows[rowIdx].cells[colIdx].style ?? {};
};
const getCellsByBorderType = (draft, range, type) => {
    const cells = getAllSelectedCells(range, draft.merges).filter(([rowIdx, colIdx]) => {
        switch (type) {
            case BorderType.BORDER_LEFT:
                return colIdx === range[0][1];
            case BorderType.BORDER_TOP:
                return rowIdx === range[0][0];
            case BorderType.BORDER_RIGHT: {
                return (colIdx === range[1][1] ||
                    draft.merges?.find(([[msri, msci], [meri, meci]]) => rowIdx >= msri &&
                        rowIdx <= meri &&
                        colIdx >= msci &&
                        colIdx <= meci)?.[1]?.[1] === range[1][1]);
            }
            case BorderType.BORDER_BOTTOM:
                return (rowIdx === range[1][0] ||
                    draft.merges?.find(([[msri, msci], [meri, meci]]) => rowIdx >= msri &&
                        rowIdx <= meri &&
                        colIdx >= msci &&
                        colIdx <= meci)?.[1]?.[0] === range[1][0]);
            default:
                return true;
        }
    });
    return cells;
};
/**
 * 设置边框 上下左右 全部和取消
 *
 * @param draft
 * @param range
 * @param style
 * @param type
 */
const setCellsByBorderPosType = (draft, range, style, type) => {
    let border;
    switch (type) {
        case BorderType.BORDER_LEFT:
            border = { borderLeft: style };
            break;
        case BorderType.BORDER_RIGHT:
            border = { borderRight: style };
            break;
        case BorderType.BORDER_TOP:
            border = { borderTop: style };
            break;
        case BorderType.BORDER_BOTTOM:
            border = { borderBottom: style };
            break;
        case BorderType.BORDER_ALL:
            border = {
                borderLeft: style,
                borderRight: style,
                borderTop: style,
                borderBottom: style,
            };
            break;
        default:
            break;
    }
    const cells = getCellsByBorderType(draft, range, type);
    cells.forEach(([rowIdx, colIdx]) => {
        ensureCellStyle(draft, rowIdx, colIdx);
        if (border) {
            draft.rows[rowIdx].cells[colIdx].style.border = {
                ...draft.rows[rowIdx].cells[colIdx].style.border,
                ...border,
            };
        }
        else {
            delete draft.rows[rowIdx].cells[colIdx].style.border;
        }
    });
};
/**
 * 水平或垂直边框
 *
 * @param draft
 * @param range
 * @param type
 * @param style
 */
const setBorderHorizontalOrVerital = (draft, range, type, style) => {
    const cells = getAllSelectedCells(range, draft.merges);
    cells.forEach(([rowIdx, colIdx]) => {
        if ((type === BorderType.BORDER_HORIZONTAL && rowIdx !== range[0][0]) ||
            (type === BorderType.BORDER_VERTICAL && colIdx !== range[0][1])) {
            ensureCellStyle(draft, rowIdx, colIdx);
            draft.rows[rowIdx].cells[colIdx].style.border = {
                ...draft.rows[rowIdx].cells[colIdx].style.border,
                [type === BorderType.BORDER_HORIZONTAL ? 'borderTop' : 'borderLeft']: style,
            };
        }
    });
};
/**
 * 外边框
 *
 * @param draft
 * @param range
 * @param style
 */
const setBorderOuter = (draft, range, style) => {
    const cells = getAllSelectedCells(range, draft.merges);
    cells.forEach(([rowIdx, colIdx]) => {
        ensureCellStyle(draft, rowIdx, colIdx);
        // 最上行
        if (rowIdx === range[0][0]) {
            draft.rows[rowIdx].cells[colIdx].style.border = {
                ...draft.rows[rowIdx].cells[colIdx].style.border,
                borderTop: style,
            };
        }
        // 最左列
        if (colIdx === range[0][1]) {
            draft.rows[rowIdx].cells[colIdx].style.border = {
                ...draft.rows[rowIdx].cells[colIdx].style.border,
                borderLeft: style,
            };
        }
        // 最右列
        if (colIdx === range[1][1] ||
            draft.merges?.find(([[msri, msci], [meri, meci]]) => rowIdx >= msri && rowIdx <= meri && colIdx >= msci && colIdx <= meci)?.[1]?.[1] === range[1][1]) {
            draft.rows[rowIdx].cells[colIdx].style.border = {
                ...draft.rows[rowIdx].cells[colIdx].style.border,
                borderRight: style,
            };
        }
        // 最下行
        if (rowIdx === range[1][0] ||
            draft.merges?.find(([[msri, msci], [meri, meci]]) => rowIdx >= msri && rowIdx <= meri && colIdx >= msci && colIdx <= meci)?.[1]?.[0] === range[1][0]) {
            draft.rows[rowIdx].cells[colIdx].style.border = {
                ...draft.rows[rowIdx].cells[colIdx].style.border,
                borderBottom: style,
            };
        }
    });
};
/**
 * 内边框
 *
 * @param draft
 * @param range
 * @param style
 */
const setBorderInner = (draft, range, style) => {
    const cells = getAllSelectedCells(range, draft.merges);
    cells.forEach(([rowIdx, colIdx]) => {
        ensureCellStyle(draft, rowIdx, colIdx);
        const border = {
            borderLeft: style,
            borderTop: style,
            borderBottom: style,
            borderRight: style,
        };
        // 最上行
        if (rowIdx === range[0][0]) {
            delete border.borderTop;
        }
        // 最左列
        if (colIdx === range[0][1]) {
            delete border.borderLeft;
        }
        // 最右列
        if (colIdx === range[1][1] ||
            draft.merges?.find(([[msri, msci], [meri, meci]]) => rowIdx >= msri && rowIdx <= meri && colIdx >= msci && colIdx <= meci)?.[1]?.[1] === range[1][1]) {
            delete border.borderRight;
        }
        // 最下行
        if (rowIdx === range[1][0] ||
            draft.merges?.find(([[msri, msci], [meri, meci]]) => rowIdx >= msri && rowIdx <= meri && colIdx >= msci && colIdx <= meci)?.[1]?.[0] === range[1][0]) {
            delete border.borderBottom;
        }
        draft.rows[rowIdx].cells[colIdx].style.border = {
            ...draft.rows[rowIdx].cells[colIdx].style.border,
            ...border,
        };
    });
};
/**
 * 设置单元格边框
 *
 * @param draft
 * @param action
 */
const setCellsBorderStyle = (draft, action) => {
    const { type, style, range } = action.payload;
    if (!range) {
        return;
    }
    switch (type) {
        case BorderType.BORDER_ALL:
        case BorderType.BORDER_LEFT:
        case BorderType.BORDER_RIGHT:
        case BorderType.BORDER_TOP:
        case BorderType.BORDER_BOTTOM:
        case BorderType.BORDER_CLEAR:
            setCellsByBorderPosType(draft, range, style, type);
            break;
        case BorderType.BORDER_HORIZONTAL:
        case BorderType.BORDER_VERTICAL:
            setBorderHorizontalOrVerital(draft, range, type, style);
            break;
        case BorderType.BORDER_OUTER:
            setBorderOuter(draft, range, style);
            break;
        case BorderType.BORDER_INNER:
            setBorderInner(draft, range, style);
            break;
        default:
            break;
    }
};
const replaceConfig = (draft, action) => {
    const { cols, rows, merges, config } = action.payload;
    draft.cols = cols;
    draft.rows = rows;
    draft.merges = merges;
    draft.config = config;
    draft.editingCell = undefined;
};
const reducer = produce((draft = defaultState, action) => {
    switch (action.type) {
        case ActionType.MERGE_CELLS:
            mergeCells(draft, action);
            break;
        case ActionType.CHANGE_ROW_HEIGHT:
            changeRowHeight(draft, action);
            break;
        case ActionType.INSERT_ROW:
            insertRow(draft, action);
            break;
        case ActionType.DELETE_ROW:
            deleteRow(draft, action);
            break;
        case ActionType.CHANGE_COL_WIDTH:
            changeColWidth(draft, action);
            break;
        case ActionType.INSERT_COL:
            insertCol(draft, action);
            break;
        case ActionType.DELETE_COL:
            deleteCol(draft, action);
            break;
        case ActionType.COPY_CELL:
            copyCell(draft, action);
            break;
        case ActionType.CUT_CELL:
            cutCell(draft, action);
            break;
        case ActionType.EDIT_CELL_VALUE:
            editCellValue(draft, action);
            break;
        case ActionType.SET_CELLS_STYLE:
            setCellsStyle(draft, action);
            break;
        case ActionType.SET_CELLS_TEXT_STYLE:
            setCellsTextStyle(draft, action);
            break;
        case ActionType.SET_CELLS_BORDER_STYLE:
            setCellsBorderStyle(draft, action);
            break;
        case ActionType.SET_PRINT_CONFIG:
            draft.config = action.payload;
            break;
        case ActionType.SET_EDITING_CELL:
            draft.editingCell = action.payload;
            break;
        case ActionType.REPLACE_CONFIG:
            replaceConfig(draft, action);
            break;
        default:
            break;
    }
});
export default reducer;
